package io.reisub.dreambot.cminer.tasks;

import org.dreambot.api.Client;
import org.dreambot.api.data.GameState;
import org.dreambot.api.methods.Calculations;
import org.dreambot.api.methods.MethodProvider;
import org.dreambot.api.methods.container.impl.Inventory;
import org.dreambot.api.methods.filter.Filter;
import org.dreambot.api.methods.interactive.Players;
import org.dreambot.api.methods.map.Tile;
import org.dreambot.api.methods.skills.Skills;
import org.dreambot.api.methods.walking.impl.Walking;
import org.dreambot.api.methods.world.Location;
import org.dreambot.api.methods.world.World;
import org.dreambot.api.methods.world.Worlds;
import org.dreambot.api.methods.worldhopper.WorldHopper;
import org.dreambot.api.script.TaskNode;
import org.dreambot.api.wrappers.interactive.Player;

import java.util.ArrayList;
import java.util.List;

public class GoToSpot extends TaskNode {
    private final Tile[] SPOTS = new Tile[]{
            new Tile(2602, 3235, 0),
            new Tile(2605, 3234, 0)
    };

    @Override
    public boolean accept() {
        Tile playerTile = Players.localPlayer().getTile();
        double closestDist = Double.MAX_VALUE;

        for (Tile t : SPOTS) {
            if (t.equals(playerTile)) return false;

            closestDist = Math.min(t.distance(), closestDist);
        }

        return !Inventory.isFull() && closestDist < 20;
    }

    @Override
    public int execute() {
        List<Tile> freeSpots = getFreeSpots();

        if (freeSpots.isEmpty()) {
            hop();
            return Calculations.random(250, 400);
        }

        Tile target = freeSpots.get(Calculations.random(0, freeSpots.size()));

        while (target.distance() > 8) {
            if (Walking.shouldWalk(Calculations.random(3, 5))) {
                Walking.walk(target);
            }

            MethodProvider.sleep(300, 500);
        }

        if (!MethodProvider.sleepUntil(() -> !Players.localPlayer().isMoving(), Calculations.random(4500, 5500))) {
            return Calculations.random(250, 400);
        }

        if (!Players.localPlayer().getTile().equals(target)) {
            Walking.walkExact(target);
            if (!MethodProvider.sleepUntil(() -> Players.localPlayer().isMoving(), Calculations.random(1000, 1500))) {
                return Calculations.random(250, 400);
            }

            MethodProvider.sleepUntil(() -> Players.localPlayer().getTile().equals(target), Calculations.random(3500, 4500));
        }

        return Calculations.random(250, 400);
    }

    private List<Tile> getFreeSpots() {
        List<Tile> freeSpots = new ArrayList<>();
        List<Player> otherPlayers = Players.all(player -> !player.getName().equals(Players.localPlayer().getName()));
        for (Tile t : SPOTS) {
            boolean taken = false;

            for (Player p : otherPlayers) {
                if (p.getTile().equals(t)) {
                    taken = true;
                    break;
                }
            }

            if (!taken) {
                freeSpots.add(t);
            }
        }

        return freeSpots;
    }

    private void hop() {
        World world = Worlds.getRandomWorld(new Filter<World>() {
            @Override
            public boolean match(World world) {
                return world.isMembers() &&
                        world.isNormal() &&
                        !world.isHighRisk() &&
                        !world.isPVP() &&
                        world.getMinimumLevel() <= Skills.getTotalLevel() &&
                        (world.getLocation().equals(Location.UK) || world.getLocation().equals(Location.GERMANY));
            }
        });

        if (world != null) {
            WorldHopper.hopWorld(world);
        }

        MethodProvider.sleepUntil(() -> Client.getGameState() == GameState.LOGGED_IN, Calculations.random(10000, 11000));
    }
}
