package io.reisub.dreambot.util;

import org.dreambot.api.input.Mouse;
import org.dreambot.api.methods.MethodContext;
import org.dreambot.api.methods.container.impl.DropPattern;
import org.dreambot.api.methods.container.impl.Inventory;
import org.dreambot.api.methods.filter.Filter;
import org.dreambot.api.wrappers.items.Item;

import java.awt.*;
import java.util.Comparator;
import java.util.List;

@SuppressWarnings("unused")
public class CInventory {
    public static final Filter<Item> foodItemFilter = item -> item.hasAction(Constants.EAT) || item.hasAction(Constants.DRINK);

    public final static DropPattern verticalDropPattern = () -> (Comparator<Item>) (o1, o2) -> {
        int o1Col = o1.getSlot() % 4;
        int o1Row = o1.getSlot() / 4;

        int o2Col = o2.getSlot() % 4;
        int o2Row = o2.getSlot() / 4;

        if (o1Col == o2Col) {
            return o1Row - o2Row;
        } else {
            return o1Col - o2Col;
        }
    };

    public final static DropPattern verticalSnakeDropPattern = () -> (Comparator<Item>) (o1, o2) -> {
        int o1Col = o1.getSlot() % 4;
        int o1Row = o1.getSlot() / 4;

        int o2Col = o2.getSlot() % 4;
        int o2Row = o2.getSlot() / 4;

        if (o1Col == o2Col) {
            if (o1Col == 0 || o1Col == 2) {
                return o1Row - o2Row;
            } else {
                return o2Row - o1Row;
            }
        } else {
            return o1Col - o2Col;
        }
    };

    public static boolean useOnNearest(String source, String... targetNames) {
        Item sourceItem = Inventory.get(source);

        List<Item> targets = Inventory.all(item -> {
            for (String targetName : targetNames) {
                if (item.getName().equals(targetName)) {
                    return true;
                }
            }

            return false;
        });

        if (sourceItem == null || targets.isEmpty()) {
            return false;
        }

        return useOnNearest(sourceItem, targets);
    }

    public static boolean useOnNearest(Item source, List<Item> targets) {
        double lowestDistance = Double.MAX_VALUE;
        Item nearest = null;

        Point sPoint = source.getDestination().getCenterPoint();

        for (Item target : targets) {
            Point tPoint = target.getDestination().getCenterPoint();

            double distance = sPoint.distance(tPoint);

            if (distance < lowestDistance) {
                lowestDistance = distance;
                nearest = target;
            }
        }

        if (nearest == null) {
            return false;
        }

        Mouse.click(source.getDestination());

        MethodContext.sleep(200, 400);

        return Mouse.click(nearest.getDestination());
    }

    public static boolean hasFood() {
        return Inventory.contains(foodItemFilter);
    }
}
