package io.reisub.dreambot.util;

import org.dreambot.api.input.Mouse;
import org.dreambot.api.input.mouse.destination.impl.MiniMapTileDestination;
import org.dreambot.api.methods.Calculations;
import org.dreambot.api.methods.MethodContext;
import org.dreambot.api.methods.interactive.Players;
import org.dreambot.api.methods.map.Tile;
import org.dreambot.api.methods.skills.Skill;
import org.dreambot.api.methods.skills.Skills;
import org.dreambot.api.wrappers.interactive.Entity;
import org.dreambot.api.wrappers.interactive.GameObject;
import org.dreambot.api.wrappers.interactive.Player;
import org.w3c.dom.css.Rect;

import java.awt.*;
import java.util.HashMap;
import java.util.Map;

@SuppressWarnings("unused")
public class Util {
    private static final String VERSION = "1.0.0";
    private static final Map<String, Rectangle> hoverMap = new HashMap<>();

    public static String getVersion() {
        return VERSION;
    }

    public static boolean playerIsIdle() {
        return playerIsIdle(0);
    }

    public static boolean playerIsIdle(long timeout) {
        Player player = Players.localPlayer();

        MethodContext.sleepWhile(() -> !player.isMoving() && !player.isAnimating(), timeout);

        return !player.isMoving() && !player.isAnimating();
    }

    public static boolean sleepUntilMoving() {
        return sleepUntilMoving(Calculations.random(3000, 3500));
    }

    public static boolean sleepUntilMoving(long timeout) {
        long start = System.currentTimeMillis();
        Player player = Players.localPlayer();

        return MethodContext.sleepUntil(player::isMoving, timeout);
    }

    public static boolean sleepUntilAnimating() {
        return sleepUntilAnimating(Calculations.random(3000, 3500));
    }

    public static boolean sleepUntilAnimating(long timeout) {
        long start = System.currentTimeMillis();
        Player player = Players.localPlayer();

        return MethodContext.sleepUntil(player::isAnimating, timeout);
    }

    public static boolean sleepUntilMovingOrAnimating() {
        return sleepUntilMovingOrAnimating(Calculations.random(3000, 3500));
    }

    public static boolean sleepUntilMovingOrAnimating(long timeout) {
        long start = System.currentTimeMillis();
        Player player = Players.localPlayer();

        return MethodContext.sleepUntil(() -> player.isMoving() || player.isAnimating(), timeout);
    }

    public static boolean sleepUntilMovingAndAnimating() {
        return sleepUntilMovingAndAnimating(Calculations.random(5000, 5500));
    }

    public static boolean sleepUntilMovingAndAnimating(long timeout) {
        long start = System.currentTimeMillis();

        if (!sleepUntilMoving(timeout)) {
            return false;
        }

        if (!MethodContext.sleepWhile(() -> Players.localPlayer().isMoving(), timeout - (System.currentTimeMillis() - start))) {
            return false;
        }

        return sleepUntilAnimating(timeout - (System.currentTimeMillis() - start));
    }

    public static int getCurrentHP() {
        return Skills.getBoostedLevels(Skill.HITPOINTS);
    }

    public static boolean isLookingAt(GameObject o) {
        Tile playerTile = Players.localPlayer().getTile();
        int px = playerTile.getX();
        int py = playerTile.getY();

        Tile objectTile = o.getTile();
        int ox = objectTile.getX();
        int oy = objectTile.getY();

        switch (Players.localPlayer().getOrientation() / 256) {
            case 0: // south
                return oy < py;
            case 1: // south-west
                return oy <= py && ox <= px;
            case 2: // west
                return ox < px;
            case 3: //north-west
                return oy >= py && ox <= px;
            case 4: // north
                return oy > py;
            case 5: // north-east
                return oy >= py && ox >= px;
            case 6: // east
                return ox > px;
            case 7: //south-east
                return oy <= py && ox >= px;
        }

        return false;
    }

    public static Rectangle getHoverEntry(String name) {
        return hoverMap.get(name);
    }

    public static void addHoverEntry(String name, Rectangle dest) {
        hoverMap.putIfAbsent(name, dest);
    }

    public static void addHoverEntry(String name, Tile dest, boolean minimap) {
        Rectangle rect;

        if (minimap) {
            MiniMapTileDestination td = new MiniMapTileDestination(dest);
            rect = td.getBoundingBox();
        } else {
            rect = dest.getTileReference().getBoundaryObject().getBoundingBox();
        }

        addHoverEntry(name, rect);
    }

    public static void addHoverEntry(String name, Tile dest) {
        addHoverEntry(name, dest, false);
    }

    public static void addHoverEntry(String name, Entity dest) {
        addHoverEntry(name, dest.getBoundingBox());
    }

    public static void hover(String name) {
        Rectangle dest = getHoverEntry(name);
        if (dest != null) {
            Mouse.move(dest);
        }
    }
}
